-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...) ---@type TSM
local Vendoring = TSM.Operations:NewPackage("Vendoring")
local L = TSM.Include("Locale").GetTable()
local TempTable = TSM.LibTSMUtil:Include("BaseType.TempTable")
local Operation = TSM.LibTSMTypes:Include("Operation")
local private = {}



-- ============================================================================
-- Module Functions
-- ============================================================================

function Vendoring.OnInitialize()
	local operationType = Operation.NewType("Vendoring", L["Vendoring"], 1)
		:SetDescriptionFunc(private.GetDescription)
		:AddNumberSetting("sellAfterExpired", 20)
		:AddBooleanSetting("sellSoulbound")
		:AddNumberSetting("keepQty")
		:AddNumberSetting("restockQty")
		:AddTableSetting("restockSources", { alts = false, ah = false, bank = false, guild = false, alts_ah = false, mail = false })
		:AddBooleanSetting("enableBuy", true)
		:AddBooleanSetting("enableSell", true)
		:AddStringSetting("vsMarketValue", "dbmarket")
		:AddStringSetting("vsMaxMarketValue", "0c")
		:AddStringSetting("vsDestroyValue", "dbmarket")
		:AddStringSetting("vsMaxDestroyValue", "0c")
	TSM.Operations.RegisterType(operationType)
end



-- ============================================================================
-- Private Helper Functions
-- ============================================================================

function private.GetDescription(operationSettings)
	local parts = TempTable.Acquire()
	if operationSettings.enableBuy and operationSettings.restockQty > 0 then
		tinsert(parts, format(L["Restocking to %d."], operationSettings.restockQty))
	end

	if operationSettings.enableSell then
		if operationSettings.keepQty > 0 then
			tinsert(parts, format(L["Keeping %d."], operationSettings.keepQty))
		end
		if operationSettings.sellSoulbound then
			tinsert(parts, L["Selling soulbound items."])
		end
	end

	local result = table.concat(parts, " ")
	TempTable.Release(parts)
	return result
end
